#!/usr/bin/perl -w 
############################## check_snmp_load #################
my $Version='1.0';
# Date : Oct 20 2014
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use RRDs;
 use Data::Dumper;
my $PROGNAME='get_snmp_interface.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $ifindex;
my $path='/tmp/';
my $warning='70';
my $critical='80';
my $speed=1000000000; # default 1000000000 = 1Gb/s
my $detectspeed="auto";
my $oid_outoctet='1.3.6.1.2.1.2.2.1.16';
my $oid_outHCoctet='1.3.6.1.2.1.31.1.1.1.10';
my $oid_inoctet='1.3.6.1.2.1.2.2.1.10';
my $oid_inHCoctet='1.3.6.1.2.1.31.1.1.1.6';
my $oid_speed='1.3.6.1.2.1.2.2.1.5';
my $oid_descr = "1.3.6.1.2.1.2.2.1.2";
my $oid_operstatus = "1.3.6.1.2.1.2.2.1.8";
my $oid_counter_type='32bit';
my $oid_highspeed='1.3.6.1.2.1.31.1.1.1.15';
sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning value in % (default: 70)\n";
	print " -c                  critical value in % (default: 80)\n";
	print " -p                  path of rrd's file\n";
	print " -s 		    vitesse de l'interface maximun en Mb/s(par défaut : autodétection)";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -i ifindex [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
sub calc_bandwith_usage
{
	my $filename =  $_[0];
	my $actual_outoctet =  $_[1];
	my $actual_inoctet =  $_[2];
	my $timestamp = time();
	my $out_ocps=0;
	my $in_ocps=0;
	my $out_oc=0;
	my $in_oc=0;
	my $inter=0;
	#print "$filename\n";
	#on test si le fichier RRD existe
	if (! -e $filename)
	{
		open(FILECOUNTER,"+>$filename");
		print FILECOUNTER "$timestamp:$actual_outoctet:$actual_inoctet: ";
		close(FILECOUNTER);
	}
	else
	{
		open(FILECOUNTERS,"+<$filename") or die "Can't open $filename: $!";
		my $saveline="";
		while (my $line = <FILECOUNTERS>)
		{
			my @value = split(':',$line);
			$inter = ($timestamp -$value[0] );
			if($inter > 10)
			{
				$out_oc=($actual_outoctet -$value[1]);
				$in_oc=($actual_inoctet - $value[2]);
			
				if($out_oc>=0 && $in_oc >=0)
				{
					$out_ocps = sprintf("%.3f", ($actual_outoctet-$value[1])/$inter);
					$in_ocps = sprintf("%.3f", ($actual_inoctet-$value[2])/$inter);
				}
				last;
			}
			else
			{
				$saveline = $line;
			}
			
		}
		seek (FILECOUNTERS,0,0);
		print FILECOUNTERS "$timestamp:$actual_outoctet:$actual_inoctet:\n";
		print FILECOUNTERS "$saveline";
		close(FILECOUNTERS);
	}
	return ($out_oc,$in_oc,$out_ocps,$in_ocps);
}
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"i=i" => \$ifindex,"ifindex=i" => \$ifindex,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"p=s" => \$path,"path=s" => \$path,
	"s=s" => \$detectspeed,"speed=s"=>\$detectspeed
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>3,
        -retries=>5
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>3,
        -retries=>5
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_descr.$ifindex");
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community,
		    -timeout=>3,
            -retries=>5
		);
		$result = $snmp->get_request("$oid_descr.$ifindex");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}


my $name =  $$result{"$oid_descr.$ifindex"};
my $la =chop($name);
if($la =~ m/[0-9a-zA-Z]$/)
{
        chomp($la);
      $name = $name.$la;
}

my $defaultspeed='';
#récupération de la vitesse de l'interface
if($detectspeed eq 'auto')
{
	$result = $snmp->get_request("$oid_speed.$ifindex");
	if($$result{"$oid_speed.$ifindex"}!=0)
	{
		$speed =  $$result{"$oid_speed.$ifindex"};
		if($speed > 4294967200)
		{
			$result = $snmp->get_request("$oid_highspeed.$ifindex");
			$speed =  $$result{"$oid_highspeed.$ifindex"};
			$speed = $speed * 1000000;
		}
	}
	else
	{
		$defaultspeed='(speed recovery failed, use default speed)';
	}
}
else
{
	$speed = $detectspeed*1000*1000 ;
}

#récupération du nombre d'octet en sortie avec le counter64 (ifHCOutOctets) si échoué utilise le counter32 (1.3.6.1.2.1.2.2.1.16)
#récupère le nombre d'octet en entré suivant la valeur du dessus
my $outoctet = 0;
my $inoctet = 0;
$result = $snmp->get_request("$oid_outHCoctet.$ifindex");
if (!defined($result)) 
{
	$result = $snmp->get_request("$oid_outoctet.$ifindex");
	$outoctet =  $$result{"$oid_outoctet.$ifindex"};
	$result = $snmp->get_request("$oid_inoctet.$ifindex");
	$inoctet =  $$result{"$oid_inoctet.$ifindex"};
}
#certain implémente un counter64 mais le laisse à 0 (ex: certain switch TP-LINK)
elsif($$result{"$oid_outHCoctet.$ifindex"}==0)
{
	$result = $snmp->get_request("$oid_outoctet.$ifindex");
	$outoctet =  $$result{"$oid_outoctet.$ifindex"};
	$result = $snmp->get_request("$oid_inoctet.$ifindex");
	$inoctet =  $$result{"$oid_inoctet.$ifindex"};
}
else
{
	#OK le noeud est full compatible avec les compteurs 64 bit
	$outoctet =  $$result{"$oid_outHCoctet.$ifindex"};
	$result = $snmp->get_request("$oid_inHCoctet.$ifindex");
	$inoctet =  $$result{"$oid_inHCoctet.$ifindex"};
	$oid_counter_type='64bit';
}

my $timestamp = time();	
my $filename = $path.$hostname.'_'.$ifindex.".csv";
	
#calcul de la bande passante utilisée
my ($out,$in,$outkbs,$inkbs) = calc_bandwith_usage($filename,$outoctet,$inoctet);

$in = $inkbs*8;
$out = $outkbs*8;
#calcul de congestion
if($in > $speed)
{
        #print "CONGESTION\n";
        my $maxspeed=0;
        if($speed < 100000000)#100Mbs
        {
            $maxspeed = $speed + ($speed/2);
        } 
        elsif($speed < 1000000000)#1000Mbs
        {
            $maxspeed = $speed + ($speed/10);
        } 
        elsif($speed < 10000000000)#10000Mbs
        {
            $maxspeed = $speed + ($speed/100);
        } 
        # print "CONGESTION: $in < $maxspeed $speed?\n";
        if($in <= $maxspeed)
        {
         #   print "CONGESTION: mise au maximun\n";
            $in = $speed;
            $inkbs = $speed;
        }
}
if($out > $speed)
{
    my $maxspeed=0;
    if($speed < 100000000) #100Mbs
    {
        $maxspeed = $speed + ($speed/2);
    } 
    elsif($speed < 1000000000)#1000Mbs
    {
        $maxspeed = $speed + ($speed/10);
    } 
    elsif($speed < 10000000000)#10000Mbs
    {
        $maxspeed = $speed + ($speed/100);
    } 
    if($out <= $maxspeed)
    {
        $out = $speed;
        $outkbs = $speed;
    }
}
$inkbs = sprintf("%.2f", ($inkbs/1000)*8);
$outkbs = sprintf("%.2f", ($outkbs/1000)*8);

my $intunity ="kb/s";
my $outunity="kb/s";
# on regarde l'unité d'affiche en kb/s ou en Mb/S
if( int($inkbs/1000)>0)
{
	$inkbs = sprintf("%.2f", $inkbs/1000);
	$intunity ="Mb/s";
}
if( int($outkbs/1000)>0)
{
    $outkbs = sprintf("%.2f", $outkbs/1000);
    $outunity ="Mb/s";
}

# teste pour savoir si on est en warning ou en critique.
my $printspeed = $speed/1000/1000;
my $warningspeed = ($speed/100)*$warning;
my $criticalspeed = ($speed/100)*$critical;
my $graphdata = "|data:in=".$in."data:out=$out:warning:$warningspeed:critical:$criticalspeed";
if($in >= $warningspeed)
{
	if($in >= ($speed/100*$critical))
	{
		print "CRITICAL $name: in=$inkbs $intunity > $critical% out=$outkbs $outunity speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"CRITICAL"};
	}
	else
	{
		print "WARNING $name: in=$inkbs $intunity > $warning% out=$outkbs $outunity speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"WARNING"};
	}
}
elsif($out >= ($speed/100*$warning))
{
	if($out >= ($speed/100*$critical))
	{
		print "CRITICAL $name: in=$inkbs $intunity out=$outkbs $outunity  > $critical% speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"CRITICAL"};
	}
	else
	{
		print "WARNING $name: in=$inkbs $intunity out=$outkbs $outunity > $warning% speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
		exit $ERRORS{"WARNING"};
	}
}
else
{
	print "OK $name: in=$inkbs $intunity out=$outkbs $outunity speed: $printspeed Mb/s $defaultspeed Counter: $oid_counter_type $graphdata\n";
	exit $ERRORS{"OK"};
}

